import json
from typing import TextIO

from src.loader import loads


def dump(
    obj: dict,
    fp: TextIO,
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: json.JSONEncoder = None,
    indent=None,
    separators=None,
    default=None,
    sort_keys: bool = False
):
    json.dump(
        obj, fp, skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular, allow_nan=allow_nan,
        cls=cls, indent=indent, separators=separators, default=default, sort_keys=sort_keys
    )


def dumps(
    s: str,
    fp: TextIO,
    *,
    skipkeys: bool = False,
    ensure_ascii: bool = True,
    check_circular: bool = True,
    allow_nan: bool = True,
    cls: json.JSONEncoder = None,
    indent=None,
    separators=None,
    default=None,
    sort_keys: bool = False
):
    obj = loads(s)
    with fp:
        dump(
            obj, fp, skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular, allow_nan=allow_nan,
            cls=cls, indent=indent, separators=separators, default=default, sort_keys=sort_keys
        )



import json
import unittest

from src.dumper import dump, dumps


class TestDump(unittest.TestCase):
    def test_dump(self):
        with open("tests/dump.json", "w") as f:
            dump({"a": "bcd // Testing"}, f)

    def test_dumps(self):
        with open("tests/dumps.json", "w") as f:
            dumps("{'a': \"bcd // Testing\"} // Testing", f)

    def test_equiv(self):
        with open("tests/dump.json") as dump_file:
            with open("tests/dumps.json") as dumps_file:
                dumps_object = json.load(dumps_file)
            dump_object = json.load(dump_file)
        self.assertDictEqual(dump_object, dumps_object)



from src.default_processing import (
    process_float, process_int, process_constant, process_object_hook,
    process_object_pairs_hook,
)


class Decoder:
    parse_float = staticmethod(process_float)
    parse_int = staticmethod(process_int)
    parse_constant = staticmethod(process_constant)
    object_hook = staticmethod(process_object_hook)
    object_pairs_hook = staticmethod(process_object_pairs_hook)



import json
import warnings
from typing import Any

from src.default_processing import (
    process_constant, process_float, process_int, process_object_hook, process_object_pairs_hook,
)


class Encoder(json.JSONEncoder):
    item_separator = ', '
    key_separator = ': '

    def __init__(
        self, *, skipkeys=False, ensure_ascii=True,
        check_circular=True, allow_nan=True, sort_keys=False,
        indent=None, separators=None, default=None
    ):
        super().__init__(
            skipkeys=skipkeys, ensure_ascii=ensure_ascii, check_circular=check_circular, allow_nan=allow_nan,
            sort_keys=sort_keys, indent=indent, separators=separators, default=default
        )

    def default(self, o: Any) -> Any:
        warnings.warn(f"Object of type {o.__class__.__name__} cannot be serialized into JSON")

    parse_float = staticmethod(process_float)
    parse_int = staticmethod(process_int)
    parse_constant = staticmethod(process_constant)
    object_hook = staticmethod(process_object_hook)
    object_pairs_hook = staticmethod(process_object_pairs_hook)



from typing import Any, Callable, Dict

from src.decoder import Decoder

GLOBALS = {
    "__builtins__": {
        "true"    : True,
        "false"   : False,
        "null"    : None,
        "Infinity": float("inf"),
        "NaN"     : float("nan"),
    }
}


def parse(
    string,
    cls: Decoder,
    object_hook: Callable,
    parse_float: Callable,
    parse_int: Callable,
    parse_constant: Callable,
    object_pairs_hook: Callable,
) -> Dict[str, Any]:
    try:
        byte_code = compile(string, "<json>", mode="eval")
        json_object: Dict[str, Any] = eval(byte_code, GLOBALS)

        if cls:
            object_hook = cls.object_hook
            parse_float = cls.parse_float
            parse_int = cls.parse_int
            parse_constant = cls.parse_constant
            object_pairs_hook = cls.object_pairs_hook

        for key, value in json_object.items():  # Apply hooks
            if isinstance(value, dict):
                json_object[key] = object_hook(value)
                object_pairs_hook(value)
            elif value in [float("-inf"), float("inf"), float("nan")]:
                json_object[key] = parse_constant(value)
            elif isinstance(value, int):
                json_object[key] = parse_int(value)
            elif isinstance(value, float):
                json_object[key] = parse_float(value)
        return json_object
    except Exception:
        raise ValueError("Unable to parse JSON content")



def process_float(item: float):
    return float(item)


def process_int(item: int):
    return int(item)


def process_constant(item: float):
    return float(item)


def process_object_hook(item: dict):
    return item


def process_object_pairs_hook(item: dict):
    values = []
    for key, value in item.items():
        values.append((key, value))



# Adapted from: https://github.com/linjackson78/jstyleson
# There may be performance suffer backtracking the last comma
def _remove_last_comma(str_list, before_index):
    i = before_index - 1
    while not str_list[i].strip():
        i -= 1

    # This is the first none space char before before_index
    if str_list[i] == ',':
        str_list[i] = ''


def convert_standard(json_str):
    """
    Convert JSON 5 to the preferred form
    - Clear all comments in json_str.
    - Removes the trailing commas
    Clear JS-style comments like // and /**/ in json_str.
    Accept a str or unicode as input.
    Args:
        json_str: A json string of str or unicode to clean up comment
    Returns:
        str: The str without comments (or unicode if you pass in unicode)
    """
    result_str = list(json_str)
    escaped = False
    normal = True
    sl_comment = False
    ml_comment = False
    quoted = False

    a_step_from_comment = False
    a_step_from_comment_away = False

    former_index = None

    for index, char in enumerate(json_str):
        if escaped:  # We have just met a '\'
            escaped = False
            continue

        if a_step_from_comment:  # We have just met a '/'
            if char != '/' and char != '*':
                a_step_from_comment = False
                normal = True
                continue

        if a_step_from_comment_away:  # We have just met a '*'
            if char != '/':
                a_step_from_comment_away = False

        if char == '"':
            if normal and not escaped:
                # We are now in a string
                quoted = True
                normal = False
            elif quoted and not escaped:
                # We are now out of a string
                quoted = False
                normal = True

        elif char == '\\':
            # '\' should not take effect in comment
            if normal or quoted:
                escaped = True

        elif char == '/':
            if a_step_from_comment:
                # Now we are in single line comment
                a_step_from_comment = False
                sl_comment = True
                normal = False
                former_index = index - 1
            elif a_step_from_comment_away:
                # Now we are out of comment
                a_step_from_comment_away = False
                normal = True
                ml_comment = False
                for i in range(former_index, index + 1):
                    result_str[i] = ""

            elif normal:
                # Now we are just one step away from comment
                a_step_from_comment = True
                normal = False

        elif char == '*':
            if a_step_from_comment:
                # We are now in multi-line comment
                a_step_from_comment = False
                ml_comment = True
                normal = False
                former_index = index - 1
            elif ml_comment:
                a_step_from_comment_away = True
        elif char == '\n':
            if sl_comment:
                sl_comment = False
                normal = True
                for i in range(former_index, index + 1):
                    result_str[i] = ""
        elif char == ']' or char == '}':
            if normal:
                _remove_last_comma(result_str, index)

    #  To remove single line comment which is the last line of json
    if sl_comment:
        for i in range(former_index, len(json_str)):
            result_str[i] = ""

    # Show respect to original input if we are in python2
    return ("" if isinstance(json_str, str) else u"").join(result_str)



import unittest
from src.loader import load, loads

json_from_load_ = {}


class TestLoad(unittest.TestCase):
    def test_load(self):
        global json_from_load_
        with open("tests/sample.json") as f:
            json_from_load = load(f)

        self.assertIsInstance(json_from_load, dict)
        json_from_load_ = json_from_load

    def test_loads(self):
        with open("tests/sample.json") as f:
            json_from_loads = loads(f.read())

        self.assertIsInstance(json_from_loads, dict)
        self.assertDictEqual(json_from_loads, json_from_load_)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['src', 'tests']

package_data = \
{'': ['*']}

modules = \
['__init__', 'README']
setup_kwargs = {
    'name': 'json5rw',
    'version': '0.2.1',
    'description': 'A tool for JSON5 I/O',
    'long_description': '# `json5rw` - A JSON5 parsing tool\n\nThis tool supports loading and dumping JSON 5 content\n\n## Supported syntax\n\nCurrently, the following are supported:\n\n- Comments\n    - `//`\n    - `/**/`\n- Different numerals\n    - Hex (`0xABC`)\n    - Int (`100`)\n    - Float (`12.34`)\n    - NaN (`NaN`, `nan`)\n    - Infinity (`Infinity`, `-Infinity`)\n- Trailing commas\n\n*Note: Unquoted keys are not supported*',
    'author': 'Andy Zhang',
    'author_email': 'andy@nwsoft.tk',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'py_modules': modules,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



from typing import Callable, TextIO

from src.decoder import Decoder
from src.parser import (
    parse,
)
from src.default_processing import (
    process_constant, process_float, process_int, process_object_hook,
    process_object_pairs_hook,
)
from src.convert_standard import convert_standard


def load(
    fp: TextIO,
    *,
    object_hook: Callable = process_object_hook,
    parse_float: Callable = process_float,
    parse_int: Callable = process_int,
    parse_constant: Callable = process_constant,
    object_pairs_hook: Callable = process_object_pairs_hook
):
    with fp:
        content = fp.read()
        content = convert_standard(content)
        return parse(content, Decoder(), object_hook, parse_float, parse_int, parse_constant, object_pairs_hook)


def loads(
    s: str,
    *,
    object_hook: Callable = process_object_hook,
    parse_float: Callable = process_float,
    parse_int: Callable = process_int,
    parse_constant: Callable = process_constant,
    object_pairs_hook: Callable = process_object_pairs_hook
):
    s = convert_standard(s)
    return parse(s, Decoder(), object_hook, parse_float, parse_int, parse_constant, object_pairs_hook)
